
#include "widget.h"
#include "../fileops.h"
#include "../econfig.h"

ImageDat *LoadImage(char *file)
{
   ImlibImage *im;
   char s1[FILEPATH_LEN_MAX], s2[FILEPATH_LEN_MAX];
   int crop_x, crop_y, crop_w = 0, crop_h = 0;
   ImageDat *image = Emalloc(sizeof(ImageDat));

   EDBUG(6, "LoadImage");
   splitstring(s1, s2, file);
   if (file[0] != '/') {
      char temp[FILEPATH_LEN_MAX];
      char *path = cwd();

      sprintf(temp, "%s/%s", path, s1);
      strcpy(s1, temp);
      Efree(path);
   }
   if (s2[0])
      getgeom(s2, &crop_x, &crop_y, &crop_w, &crop_h);
   if ((im = Imlib_load_image(imlib, s1)) == NULL) {
      fprintf(stderr, "Unable to load image %s.\n", s1);
      exit(-1);
   }
   if (crop_w && crop_h)
      Imlib_crop_image(imlib, im, crop_x, crop_y, crop_w, crop_h);
   Imlib_set_image_shape(imlib, im, &imask);
   Imlib_render(imlib, im, im->rgb_width, im->rgb_height);
   image->im = im;
   image->w = im->rgb_width;
   image->h = im->rgb_height;
   image->name = duplicate(s1);
   EDBUG_RETURN(image);
}

void FreeImageDat(ImageDat * dat)
{
   EDBUG(5, "FreeImageDat");
   if (!dat)
      EDBUG_RETURN_;
   Efree(dat->name);
   dat->name = NULL;
   Imlib_kill_image(imlib, dat->im);
   Efree(dat);
   dat = NULL;
   EDBUG_RETURN_;
}

void LoadSingleImageList(ImageList *list, int num)
{
   char s1[FILEPATH_LEN_MAX];
  
   if (list->images[num])
      return;   
   if (list->middle_peak) {
      int i;
      
      if (num > (list->number / 2))
         i = num - (list->number / 2);
      else
         i = (list->number / 2) - num;
      if (list->ow < list->oh)
         sprintf(s1, "%s %d %d %d %d", list->imagename, list->x, list->y + 
                 list->delta * i, list->w, list->h);
      else
         sprintf(s1, "%s %d %d %d %d", list->imagename, list->x + list->delta * 
                 i, list->y, list->w, list->h);
      list->images[(list->number / 2) - i] = 
      list->images[(list->number / 2) + i] = LoadImage(s1);
      num = (list->number / 2) - i;
   }
   else {
      num %= list->number;
      if (list->ow < list->oh)
         sprintf(s1, "%s %d %d %d %d", list->imagename, list->x, list->y + 
                 list->delta * num, list->w, list->h);
      else
         sprintf(s1, "%s %d %d %d %d", list->imagename, list->x + list->delta * 
                 num, list->y, list->w, list->h);
      list->images[num] = LoadImage(s1);
   }
   if ((list->scale_w != 1.0) || (list->scale_h != 1.0)) {
      list->images[num]->w *= list->scale_w;
      list->images[num]->h *= list->scale_h;
      Imlib_render(imlib, list->images[num]->im, list->images[num]->w, list->images[num]->h);
   }
}
   
ImageList *ConfigImageList(FILE * file)
{
   char st[FILEPATH_LEN_MAX];
   char s1[FILEPATH_LEN_MAX], s2[FILEPATH_LEN_MAX];
   int end = FALSE;
   ImageList *list = Emalloc(sizeof(ImageList));

   EDBUG(5, "ConfigImageList");
   list->number = 0;
   list->images = NULL;
   list->delta = 0;
   list->middle_peak = FALSE;
   list->scale_w = list->scale_h = 1.0;
   while (!end) {
      if (GetNextLine(st, file)) {
         splitstring(s1, s2, st);
         upstr(s1);
         if (!strncmp("END", s1, 3))
            end = TRUE;
         else if (!strncmp("NUMBER", s1, 6)) {
            int i;

            list->number = atoi(s2);
            list->images = Emalloc((int)(list->number * sizeof(ImageList)));
            for (i = 0; i < list->number; i++)
               list->images[i] = NULL;
         }
         else if (!strncmp("MIDDLE_PEAK", s1, 11))
            list->middle_peak = parsebool(s2);
         else if (!strncmp("DELTA", s1, 5))
            list->delta = atoi(s2);
         else if (!strncmp("IMAGE", s1, 5)) {
            char temp_s1[4096];
            char temp_s2[4096];
            ImageDat *temp_im = NULL;
            char *path;
            
            if (!list->number) {
               fprintf(stderr, "number must preceed images in ImageList\n");
               exit(-1);
            }
            if (!list->delta) {
               fprintf(stderr, "unknown size (delta) in ImageList\n");
               exit(-1);
            }
            splitstring(temp_s1, temp_s2, s2);
            getgeom(temp_s2, &list->x, &list->y, &list->w, &list->h);
            temp_im = LoadImage(temp_s1);
            list->ow = temp_im->w;
            list->oh = temp_im->h;
            FreeImageDat(temp_im);
            path = cwd();
            sprintf(temp_s2, "%s/%s", path, temp_s1);
            Efree(path);
            list->imagename = duplicate(temp_s2);
         }
         else
            fprintf(stderr, "Unknown line in ImageList config\n");
      }
      else
         end = TRUE;
   }
   EDBUG_RETURN(list);
}

void ScaleImageList(ImageList * list, double scale_w, double scale_h)
{
   int x;

   EDBUG(5, "ScaleImageList");
   list->scale_w = scale_w;
   list->scale_h = scale_h;
   for (x = 0; x < list->number; x++) {
      if (list->images[x]) {
         list->images[x]->w *= scale_w;
         list->images[x]->h *= scale_h;
         Imlib_render(imlib, list->images[x]->im, list->images[x]->w,
                      list->images[x]->h);
      }
   }
   EDBUG_RETURN_;
}

void FreeImageList(ImageList * list)
{
   int i;
   EDBUG(5, "FreeImageList");
   if (!list)
      EDBUG_RETURN_;
   for (i = 0; i < list->number; i++)
      if (list->images[i])
         FreeImageDat(list->images[i]);
   Efree(list->images);
   Efree(list);
   list = NULL;
   EDBUG_RETURN_;
}
